@@ -5,7 +5,7 @@ package HTML::Mason;
 
 use 5.005;
 
-$HTML::Mason::VERSION = '1.18';
+$HTML::Mason::VERSION = '1.20';
 
 use HTML::Mason::Interp;
 
@@ -147,7 +147,7 @@ reference companion to the Developer's Manual is the Request API
 documentation, L<HTML::Mason::Request|HTML::Mason::Request>.
 
 Whoever is responsible for setting up and tuning Mason should read the
-(L<Administrator's Manual|HTML::Mason::Admin>), though developers
+L<Administrator's Manual|HTML::Mason::Admin>, though developers
 will also benefit from reading it as well. This document covers more
 advanced configuration scenarios and performance optimization. The
 reference companion to the Administrator's manual is the
