@@ -120,6 +120,12 @@ sub output
     return ${$self->{output}};
 }
 
+sub remove_filter
+{
+    my $self = shift;
+    delete $self->{filter_from};
+}
+
 # Makes a reference to the current output and stores in
 # $self->{output}.  Unless there are filters, this will be a reference
 # to the same string as $self->{buffer}.
@@ -133,8 +139,10 @@ sub _make_output
     
     $self->{output} = $self->{buffer};
     if ($self->{filter_from}) {
-	my $filtered = $self->{filter_from}->filter->(${$self->{output}});
-	$self->{output} = \$filtered;
+        if ( my $filter = $self->{filter_from}->filter ) {
+            my $filtered = $filter->( ${ $self->{output} } );
+            $self->{output} = \$filtered;
+        }
     }
 }
 
