@@ -147,8 +147,8 @@ sub compiled_component
     my ($self, %p) = @_;
     my $obj_text = '';
 
-    $self->{compiled_def} = $self->_compile_subcomponents if %{ $self->{def} };
-    $self->{compiled_method} = $self->_compile_methods if %{ $self->{method} };
+    local $self->{compiled_def} = $self->_compile_subcomponents if %{ $self->{def} };
+    local $self->{compiled_method} = $self->_compile_methods if %{ $self->{method} };
 
     $self->{current_comp} = $self;
 
@@ -362,6 +362,7 @@ EOF
     }
 
     return join '', ( $self->preamble,
+                      $self->_set_request,
 		      "my \%ARGS;\n",
 		      @args,
                       $self->_filter,
@@ -374,6 +375,15 @@ EOF
 		    );
 }
 
+sub _set_request
+{
+    my $self = shift;
+
+    return if $self->in_package eq 'HTML::Mason::Commands';
+
+    return 'local $' . $self->in_package . '::m = $HTML::Mason::Commands::m;' . "\n";
+}
+
 my %coercion_funcs = ( '@' => 'HTML::Mason::Tools::coerce_to_array',
 		       '%' => 'HTML::Mason::Tools::coerce_to_hash',
 		     );
