@@ -50,8 +50,11 @@ HTML::Mason::Escapes - Functions to escape text for Mason
 
 =head1 DESCRIPTION
 
-This module contains exportable functions that are intended to be used
-by other Mason modules.
+This module contains functions for implementing Mason's L<substitution
+escaping|HTML::Mason::Devel/Escaping expressions> feature.  These
+functions may also be called directly.
+
+=over 4
 
 =item html_entities_escape
 
@@ -64,10 +67,10 @@ the section on L<User-defined Escapes in the Developer's
 Manual|HTML::Mason::Devel/User-defined Escapes> for more details on
 how to do this.
 
-=item uri_escape
+=item url_escape
 
 This takes a scalar reference and replaces any text it contains
-matching C<[^a-zA-Z0-9_.-]> with the URI-escaped equivalent, a percent
+matching C<[^a-zA-Z0-9_.-]> with the URL-escaped equivalent, a percent
 sign (%) followed by the hexadecimal number of that character.
 
 =item basic_html_escape
@@ -91,6 +94,6 @@ scripting attacks.  See
 http://www.megasecurity.org/Info/cross-site_scripting.txt for more
 details.
 
-It is not exportable.
+=back
 
 =cut
