@@ -15,7 +15,7 @@ use Getopt::Long;
 
 use vars qw($VERBOSE $DEBUG @SHARED);
 
-$VERBOSE = $ENV{MASON_DEBUG} || $ENV{MASON_VERBOSE};
+$VERBOSE = $ENV{MASON_DEBUG} || $ENV{MASON_VERBOSE} || $ENV{TEST_VERBOSE};
 $DEBUG = $ENV{MASON_DEBUG};
 
 $| = 1;
@@ -49,8 +49,8 @@ I am <% $comp->is_file_based ? '' : 'not ' %>file-based.
 % if (defined($comp->name)) {
 My short name is <% $comp->name =~ /anon/ ? '[anon something]' : $comp->name %>.
 % }
-% if ($comp->is_subcomp and defined($comp->parent_comp)) {
-My parent component is <% $comp->parent_comp->title %>.
+% if ($comp->is_subcomp and defined($comp->owner)) {
+My parent component is <% $comp->owner->title %>.
 % }
 % if (defined($comp->dir_path)) {
 My directory is <% $comp->dir_path %>.
@@ -73,6 +73,21 @@ $comp
 </%args>
 EOF
 	    },
+            { path => '/shared/display_req_obj',
+              component => <<'EOF',
+My depth is <% $m->depth %>.
+
+The top-level component is <% $m->request_comp->title %>.
+
+My stack looks like:
+-----
+% foreach my $comp ($m->callers) {
+<% $comp->title %>
+% }
+-----
+
+EOF
+            },
 	  );
 
 sub new
@@ -81,6 +96,7 @@ sub new
     my %p = @_;
 
     GetOptions( 'create' => \$p{create},
+		'tests-to-run=s' => \$p{tests_to_run},
 	      );
 
     die "No group name provided\n"
@@ -92,6 +108,9 @@ sub new
     $p{pre_test_cleanup} = 1
         unless exists $p{pre_test_cleanup};
 
+    $p{tests_to_run} = $ENV{TESTS_TO_RUN}
+        if !defined($p{tests_to_run}) and defined($ENV{TESTS_TO_RUN});
+ 
     return bless {
 		  interp_class => 'HTML::Mason::Interp',
 		  %p,
@@ -333,8 +352,17 @@ sub _run_tests
 {
     my $self = shift;
 
+    my %tests_to_run;
+    if ($self->{tests_to_run}) {
+	my @tests_to_run = sort { $a <=> $b } split(/\s*,\s*/, $self->{tests_to_run});
+	die "tests_to_run must be a list of numbers separated by commas" if grep(!/\d+/, @tests_to_run);
+	%tests_to_run = map { ($_, 1) } @tests_to_run;
+	printf ("Running only test%s %s\n", @tests_to_run == 1 ? "" : "s", join(", ", @tests_to_run));
+    }
+    
     my $count = scalar @{ $self->{tests} };
     print "\n1..$count\n";
+
     if ($VERBOSE)
     {
 	print "Running $self->{name} tests ($count tests): $self->{description}\n";
@@ -343,13 +371,15 @@ sub _run_tests
     my $x = 1;
     foreach my $test ( @{ $self->{tests} } )
     {
-	print "Running $test->{name} (#$x): $test->{description}\n"
-	    if $VERBOSE;
-
-	$self->{current_test} = $test;
-	$self->_make_component unless $test->{skip_component};
-	$self->_run_test;
-
+	if (!%tests_to_run or $tests_to_run{$x}) {
+	    print "Running $test->{name} (#$x): $test->{description}\n"
+		if $VERBOSE;
+	    
+	    $self->{current_test} = $test;
+	    $self->_make_component unless $test->{skip_component};
+	    $self->_run_test;
+	}
+	    
 	$x++;
     }
 }
@@ -459,10 +489,19 @@ sub check_output
 {
     my ($self, %p) = @_;
 
-    # Whitespace at end can vary.  (Or rather, it is varying in the tests, and
-    # should be made not to vary, but I don't have time to fix it yet.)
-    for ($p{actual}, $p{expect}) {  s/\s+$//  }
-    my $same = ($p{actual} eq $p{expect});
+    my $same;
+
+    # Allow a regex for $p{expect}
+    if (ref $p{expect}) {
+	$same = ($p{actual} =~ /$p{expect}/);
+
+    } else {
+	# Whitespace at end can vary.  (Or rather, it is varying in the tests, and
+	# should be made not to vary, but I don't have time to fix it yet.)
+	
+	for ($p{actual}, $p{expect}) {  s/\s+$//  }
+	$same = ($p{actual} eq $p{expect});
+    }
 
     if (!$same and $VERBOSE) {
 	print "Got ...\n-----\n$p{actual}\n-----\n   ... but expected ...\n-----\n$p{expect}\n-----\n";
@@ -758,6 +797,15 @@ the test harness will simply output its results.  This allows you to
 cut and paste these results back into the test file (assuming they are
 correct!).
 
+=head2 Running selected tests
+
+You can run just some of a test file with the '--tests-to-run' flag,
+or by setting the TESTS_TO_RUN environment variable. In either case
+the value is a comma-separated list of test numbers.  e.g.
+
+    perl ./01-syntax.t --tests-to-run=3,5
+    TESTS_TO_RUN=3,5 perl ./01-syntax.t
+
 =head1 SEE ALSO
 
 L<HTML::Mason|HTML::Mason>
